#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <linux/joystick.h>

#define GAMEPAD_LIMIT 10

#pragma pack(1)

typedef struct
{
    int     AxisCount;
    int     ButtonCount;
    int     DriverVersion;
    int     Filler;
    char    GamepadName[128];
} InfoBlock;


extern char* GP_GetAttachedList();
extern char* GP_FillInfoBlock( int argPadIndex, InfoBlock* argInfo );
extern char* GP_RawRead( int argPadIndex, struct js_event* argEvent );


        char myList[1000];
        char myError[1000];
        
        int myFDs[GAMEPAD_LIMIT];


//============================================================================
//============================================================================
//    I N T E R N A L   R O U T I N E S
//============================================================================
//============================================================================
int OpenGamePad( argPadIndex )
{
        char theDeviceSpec[256];
        
        sprintf( theDeviceSpec, "/dev/input/js%d", argPadIndex );
        
        int fd = open( theDeviceSpec, O_RDONLY | O_NONBLOCK );

        if( fd <= 0 ) fd = 0;
        
        return fd;

}
//============================================================================
void GetGamePadInfo( int argFD, InfoBlock* argInfo )
{
        ioctl( argFD, JSIOCGAXES,        &argInfo->AxisCount );
        ioctl( argFD, JSIOCGBUTTONS,     &argInfo->ButtonCount );
        ioctl( argFD, JSIOCGVERSION,     &argInfo->DriverVersion );
        ioctl( argFD, JSIOCGNAME( 128 ), argInfo->GamepadName );
}
//============================================================================
//============================================================================
//    G A M B A S   E N T R Y   P O I N T S
//============================================================================
//============================================================================
char* GP_GetAttachedList()
{
        int p, theFD;
        
        InfoBlock theInfo;
        
        char* theSpot = myList;
        
        *theSpot = '\0';

        for( p = 0; p < GAMEPAD_LIMIT; p++ )
        {
            int theFail = 0;
            
            if( myFDs[p] == 0 )
            {
                theFD = OpenGamePad( p );
                if( theFD > 0 )
                {
                    GetGamePadInfo( theFD, &theInfo );
                    close( theFD );
                }
                else
                {
                    theFail = 1;
                }
            }
            else
            {
                GetGamePadInfo( myFDs[p], &theInfo );
            }
            
            if( theFail == 0 )
            {
                int w = sprintf( theSpot, "%s\n", theInfo.GamepadName );
                theSpot += w;
            }
        
        }
        
        if( theSpot > myList )
        {
            theSpot[-1] = '\0';
        }

        return myList;
}
//============================================================================
char* GP_FillInfoBlock( int argPadIndex, InfoBlock* argInfo )
{
//--- Check for Valid Index Value

        if( argPadIndex < 0 ) return "Negative Index is Invalid";
        if( argPadIndex >= GAMEPAD_LIMIT ) return "Index Exceeds Limit";

//--- Get Info Preserving Open/Close State

        int theFD = 0;
        
        char* theResult = NULL;

        if( myFDs[argPadIndex] == 0 )
        {
            theFD = OpenGamePad( argPadIndex );
            if( theFD > 0 )
            {
                GetGamePadInfo( theFD, argInfo );
                close( theFD );
            }
            else
            {
                sprintf( myError, "Unable to open Gamepad %d", argPadIndex );
                theResult = myError;
            }
        }
        else
        {
            GetGamePadInfo( myFDs[argPadIndex], argInfo );
        }

//--- Exit

        return theResult;
        
}
//============================================================================
char* GP_RawRead( int argPadIndex, struct js_event* argEvent )
{
//--- Check for Valid Index Value

        if( argPadIndex < 0 ) return "Negative Index is Invalid";
        if( argPadIndex >= GAMEPAD_LIMIT ) return "Index Exceeds Limit";

//--- Make Sure GamePad is Open

        int theFD = myFDs[argPadIndex];

        if( theFD == 0 )
        {
            theFD = OpenGamePad( argPadIndex );
            
            if( theFD == 0 )
            {
                sprintf( myError, "Unable to open Gamepad %d", argPadIndex );
                return myError;
            }
            else
            {
                myFDs[argPadIndex] = theFD;
            }
        }

//--- Read from the Device FD

        int theResult = read( theFD, argEvent, sizeof( struct js_event ) );

        if( theResult < 0 )
        {
            return "Empty";
        }

//--- Exit       

        return NULL;
}
//============================================================================
//============================================================================
//    H O U S E K E E P I N G
//============================================================================
//============================================================================
__attribute__((constructor)) void init(void)
{
        int p;

        for( p = 0; p < GAMEPAD_LIMIT; p++ )
        {
            myFDs[p] = 0;
        }

}
//============================================================================
__attribute__((destructor)) void fini(void)
{
        int p;

        for( p = 0; p < GAMEPAD_LIMIT; p++ )
        {
            if( myFDs[p] != 0 )
            {
                close( myFDs[p] );
                myFDs[p] = 0;
            }
        }                
}
//============================================================================

// $ gcc -c -Wall -Werror -fpic libGambas_GamePad.c 
// $ gcc -shared -o libGambas_GamePad.so libGambas_GamePad.o



